#   Copyright (c) 1999 Boulder Real Time Technologies, Inc.           
#                                                                     
#   This software module is wholly owned by Boulder Real Time         
#   Technologies, Inc. Any use of this software module without        
#   express written permission from Boulder Real Time Technologies,   
#   Inc. is prohibited.                                               

package provide displaynoise 1.0

namespace eval displaynoise {

	variable Top		;# array of toplevel names, indexed by widget
	variable DialogTop	;# array of dialog toplevel names, indexed by widget
	variable Canvas		;# array of canvas names, indexed by widget
	variable Psfile 	;# array of Postscript file names, indexed by widget
	variable Tmin		;# array of minimum times, indexed by widget
	variable Tmax		;# array of maximum times, indexed by widget
	variable Ymin		;# array of minimum trace amplitude, indexed by widget
	variable Ymax		;# array of maximum trace amplitude, indexed by widget
	variable Fmin		;# array of minimum frequencies, indexed by widget
	variable Fmax		;# array of maximum frequencies, indexed by widget
	variable Amin		;# array of minimum amplitudes, indexed by widget
	variable Amax		;# array of maximum amplitudes, indexed by widget
	variable Rsptype	;# array of response types, indexed by widget
	variable Prsptype	;# array of plot response types, indexed by widget
	variable Db		;# array of Database pointers, indexed by widget
	variable Row		;# array of Database row indexes, indexed by widget
	variable Xlog		;# array of frequency axes log flags, indexed by widget
	variable Ylog		;# array of amplitude axes log flags, indexed by widget
	variable Scg		;# array of sta-chan trace group handles, indexed by widget
	variable Instance	;# widget instance number
	variable PrintInstance	;# print instance number
	variable VectorTag	;# array of vector tags for all vectors, indexed by widget
	variable TrplotTag	;# array of tags for trace viewports, indexed by widget
	variable AmplotTag	;# array of tags for amplitude viewports, indexed by widget
	variable TrTag		;# array of tags for trace plots, indexed by widget
	variable AmTag		;# array of tags for amplitude plots, indexed by widget
	variable Stachan	
	variable Units	

	proc create {widget db row prsptype units} {
		variable Top
		variable DialogTop
		variable Canvas
		variable Psfile
		variable Tmin
		variable Tmax
		variable Ymin
		variable Ymax
		variable Fmin
		variable Fmax
		variable Amin
		variable Amax
		variable Rsptype
		variable Prsptype
		variable Db	
		variable Row
		variable Xlog
		variable Ylog
		variable Scg
		variable PrintInstance
		variable Instance
		variable VectorTag
		variable TrplotTag
		variable AmplotTag
		variable TrTag
		variable AmTag
		variable Stachan
		variable Units

		set cheight 850
		set cwidth [expr round(double($cheight)*0.75)]
		set db [lreplace $db 3 3 $row]
		set Db($widget) $db
		set Row($widget) $row
		if {[info exists Instance] == 0} {
			set Instance 0
		} else {
			incr Instance
		}
		if {[info exists Top($widget)] == 0} {	
			set top $widget
			set Top($widget) $top
			set sta [dbgetv $db 0 $row sta]
			set chan [dbgetv $db 0 $row chan]
			set Stachan($widget) "$sta $chan"
			toplevel $top
			wm protocol $top WM_DELETE_WINDOW "displaynoise::destroy $widget"
			wm title $top [format "displaynoise%d %s" $Instance $Stachan($widget)]
			wm resizable $top false false
			set fr1 [format "%s.f1" $top]
			set can [format "%s.c" $top]
			set Canvas($widget) $can
	
			frame $fr1
	
			if {[info exists PrintInstance] == 0} {set PrintInstance 0}
	
			set Psfile($widget) [format "displaynoise_%d_%d.ps" [pid] $PrintInstance]
	
			button $fr1.done -text Dismiss -command "displaynoise::destroy $widget" -height 1 -pady 0
			button $fr1.options -text "Options..." -command "displaynoise::make_dialog $widget" -height 1 -pady 0
			button $fr1.print -text print -command "displaynoise::print $widget $can 1" -height 1 -pady 0
			button $fr1.save -text save -command "displaynoise::print $widget $can 0" -height 1 -pady 0
			entry $fr1.save_entry -textvariable displaynoise::Psfile($widget) -width 25 -font {times 11}
			grid $fr1.done -row 0 -column 0 -padx 0 -pady 0 -sticky w
			grid $fr1.options -row 0 -column 1 -padx 0 -pady 0 -sticky w
			grid $fr1.print -row 0 -column 2 -padx 0 -pady 0 -sticky e
			grid $fr1.save -row 0 -column 3 -padx 0 -pady 0 -sticky e
			grid $fr1.save_entry -row 0 -column 4 -columnspan 2 -padx 0 -pady 0 -sticky e
	
			set Tmin($widget) 0.0
			set Tmax($widget) 10.0
			set Ymin($widget) -10.0
			set Ymax($widget) 10.0
			set Fmin($widget) 0.0001
			set Fmax($widget) 10.0
			set Amin($widget) 0.0001
			set Amin($widget) 10.0
			set Rsptype($widget) displacement
			set Xlog($widget) log
			set Ylog($widget) lin
	
			grid $fr1 -row 0 -column 0 -sticky ew
			grid columnconfigure $fr1 0 -weight 0
			grid columnconfigure $fr1 1 -weight 1
			grid columnconfigure $fr1 2 -weight 0
			grid columnconfigure $fr1 3 -weight 0
			grid columnconfigure $fr1 4 -weight 0
		} else {
			set top $widget
			set fr1 [format "%s.f1" $top]
			set can [format "%s.c" $top]
			vector_free -tag $VectorTag($widget)
			if {$Scg($widget) != ""} {destroy_scg $Scg($widget)}
			destroy_dialog $widget
			::destroy $can
		}
	
		set TrplotTag($widget) [format "displaynoise_trplot_%d" $Instance]
		set AmplotTag($widget) [format "displaynoise_amplot_%d" $Instance]
		set TrTag($widget) [format "displaynoise_tr_%d" $Instance]
		set AmTag($widget) [format "displaynoise_am_%d" $Instance]
		set VectorTag($widget) [format "displaynoise_vectors_%d" $Instance]
	
		set sta [dbgetv $db 0 $row sta]
		set chan [dbgetv $db 0 $row chan]
		set twin [dbeval $db \(endtime-time\)]
		set time0 [dbgetv $db 0 $row time]
		set endtime [dbgetv $db 0 $row endtime]
		set Tmin($widget) [expr -0.1*$twin]
		set Tmax($widget) [expr 1.1*$twin]
		set time [dbeval $db strtime\(time\)]
		set duration [dbeval $db endtime-time]
		set rsptype [dbgetv $db 0 $row rsptype]
		set spectype [dbgetv $db 0 $row rsptype]
		set freqmax [dbgetv $db 0 $row freqmax]
		set fname [dbeval $db extfile\(\)]
		set dir [dbgetv $db 0 $row dir]
		if {$prsptype == 0} {set prsptype $rsptype}

		if {$units == "0"} {
			set units pdbnm
			if {$rsptype == "C"} {set units pdbuv}
		}
		set Units($widget) $units

		set calibl 0.0
		if {$units == "pdbv" || $units == "pdbuv"} {
			set db2 [dblookup $db 0 nominalresp 0 0]
			set mlist [dbmatches $db $db2 [format "hookn%d" $Instance]]
			set n [llength $mlist]
			if {$n == 1} {
				set row2 [lindex $mlist 0]
				set db2 [lreplace $db2 3 3 $row2]
				set calibl [dbgetv $db2 0 $row2 calibl]
			}
			if {$calibl == 0.0} {
				tk_messageBox -message "Cannot get calibl to convert counts to volts" -type ok
				return
			}
			if {$units == "pdbv"} {
				set calibl [expr $calibl*1.e-6]
			}
		}

		set dbname [dbquery $db dbDATABASE_FILENAME]
		set Scg($widget) ""
		set tmin [expr $time0+$Tmin($widget)]
		set tmax [expr $time0+$Tmax($widget)]
		set tmin 0.0
		set tmax 0.0
		if {[catch "create_scg [list $db] $tmin $tmax -sta $sta -chan $chan" out] == 0} {
			set Scg($widget) $out
		} else {puts $out}
	
		set cwmid [expr round(double($cwidth)*0.5)]
	
		set cttop 60
		set ctline 15
		set nlines 5
		set ctheight [expr round(double($ctline)*$nlines)]
	
		set ctrtop [expr $cttop+100]
		set ctrheight [expr round(double($cheight - 30 - $ctrtop)*0.25)]
		set cawidth [expr round(double($cwidth)*1.00)]
		set catop [expr $ctrtop + $ctrheight]
		set caheight [expr round(double($cheight - 30 - $catop))]
	
		canvas $can -width $cwidth -height $cheight -background white
		$can create viewport [format "bckg%d" $Instance] 0 $cttop -height [expr $catop+$caheight-$cttop] -width $cwidth \
				-fill_frame \#f0ffe0
	
		grid $can -row 1 -column 0 -sticky nsew -padx 0 -pady 0
		grid columnconfigure $top 0 -weight 1
	
		set display_text [format "Power spectrum processing results"]
	
		$can create text $cwmid 10 -anchor n -font {Times 20 bold} -text "$display_text"
	
		set xpos [line_split $cwidth 5]
		set x0 [expr 3]
		set xw [expr $cwidth-3]
		set xpos [expr 10]
		set xr1 $x0
		set x1 [expr $xr1+5]
		set xr2 [expr round($xr1+0.25*$xw)]
		set x2 [expr $xr2+5]
		set xr3 [expr round($xr2+0.25*$xw)]
		set x3 [expr $xr3+5]
		set xr4 [expr round($xr3+0.25*$xw)]
		set x4 [expr $xr4+5]
		set xr5 [expr round($xr4+0.25*$xw)]
		set xmid [expr round(0.5*($cwidth))]
		set xmid2 [expr round(0.3333*($cwidth))]
		set xmid3 [expr round(0.6666*($cwidth))]
		set xpos3 [expr $xmid+10]
		set xpos4 [expr $xmid+10]
		set xposa [expr $xmid2+10]
		set xposb [expr $xmid3+10]
		set xpos2a [ expr $xmid2+10]
		set xpos2b [ expr $xmid3+10]
		set cline [expr $cttop]
		$can create text $x1 $cline -anchor nw -font {Times 11} -text "station: "
		$can create text $x2 $cline -anchor nw -font {Times 11} -text "channel: "
		set y0 $cline
		set cline [expr $cline + $ctline]
		$can create text $x1 $cline -anchor nw -font {Times 11} -fill blue -text $sta
		$can create text $x2 $cline -anchor nw -font {Times 11} -fill blue -text $chan
		set cline [expr $cline + $ctline]
		$can create rectangle $xr1 $y0 $xr2 $cline -outline \#808080 -width 0
		$can create rectangle $xr2 $y0 $xr3 $cline -outline \#808080 -width 0
		$can create rectangle $xr3 $y0 $xr4 $cline -outline \#808080 -width 0
		set y0 $cline
		$can create text $x1 $cline -anchor nw -font {Times 11} -text "start time: "
		$can create text $x3 $cline -anchor nw -font {Times 11} -text "time duration: "
		set cline [expr $cline + $ctline]
		$can create text $x1 $cline -anchor nw -font {Times 11} -fill blue -text $time
		$can create text $x3 $cline -anchor nw -font {Times 11} -fill blue -text [format "%.3f sec" $duration]
		set cline [expr $cline + $ctline]
		$can create rectangle $xr1 $y0 $xr3 $cline -outline \#808080 -width 0
		$can create rectangle $xr3 $y0 $xr4 $cline -outline \#808080 -width 0

		set cline [expr $cline + $ctline]
		$can create text $x1 $cline -anchor w -font {Times 11} -text "Power: "
		$can create text $x2 $cline -anchor w -font {Times 11} -text "Power Uncertainty: "
		if {$rsptype != "C"} {
			$can create text $x3 $cline -anchor w -font {Times 11} -text "Peterson High Noise: "
			$can create text $x4 $cline -anchor w -font {Times 11} -text "Peterson Low Noise: "
		}
		set cline [expr $cline + $ctline]
		$can create line $x1 $cline [expr $xr2-20] $cline -fill darkblue -width 2
		$can create line $x2 $cline [expr $xr3-20] $cline -fill darkblue -width 0
		if {$rsptype != "C"} {
			$can create line $x3 $cline [expr $xr4-20] $cline -fill red -width 2
			$can create line $x4 $cline [expr $xr5-20] $cline -fill \#00a000 -width 2
		}
	
		set viewt [format "viewt_%d" $Instance]
		set viewa [format "viewa_%d" $Instance]
	
		set Fmin($widget) 0.001
		set Fmax($widget) $freqmax
	
		set lv [make_spectrum $widget $rsptype $fname $prsptype $units $calibl]
		set a [lindex $lv 0]
		set p [lindex $lv 1]
		if {$a == "0"} {
			tk_messageBox -message "Problem with response file $fname: Cannot parse response." -type ok
			return
		}

		switch $prsptype {
		C	{
				set Rsptype($widget) counts
			}
		A	{
				set Rsptype($widget) acceleration
			}
		V	{
				set Rsptype($widget) velocity
			}
		default	{
				set Rsptype($widget) displacement
			}
		}
		set Prsptype($widget) $Rsptype($widget)
	
		set cline [expr $catop + $caheight + 15]
		$can create text 10 $cline -anchor w -font {Helvetica 14 bold italic} -text BRTT
		$can create text 60 $cline -anchor w -font {Times 14} -text Antelope
		$can create text 150 $cline -anchor w -font {Times 11} -text [strtime [expr double([clock seconds])]]
	
		set amin 1.e20
		set amax -1.e20
		set nx [vector_get $a nx]
		set fhi [expr 0.5*$Fmax($widget)]
		loop i 0 $nx {
			set xy [vector_get $a xy $i]
			set freq [lindex $xy 0]
			set amp [lindex $xy 1]
			if {$i == 0} {set fmin $freq; set flo [expr 2.0*$freq]}
			if {$freq < $flo} {continue}
			if {$freq > $fhi} {break}
			if {$amp > $amax} {set amax $amp}
			if {$amp < $amin} {set amin $amp}
		}
		if {$rsptype != "C"} {
			set nx [vector_get $p nx]
			loop i 0 $nx {
				set xy [vector_get $p xy $i]
				set freq [lindex $xy 0]
				set high [lindex $xy 1]
				set low [lindex $xy 2]
				if {$high > $amax} {set amax $high}
				if {$high < $amin} {set amin $high}
				if {$low > $amax} {set amax $low}
				if {$low < $amin} {set amin $low}
			}
		}

		set Fmin($widget) $fmin
		set Amin($widget) [expr $amin - 10.0]
		set Amax($widget) [expr $amax + 10.0]
	
		$can create viewport $viewt 0 $ctrtop -height $ctrheight -width $cawidth -tag $TrplotTag($widget) -fill \#000080 \
				-fill_frame \#f0f0ff \
				-xmode time -x0 $time0 -xleft $Tmin($widget) -xright $Tmax($widget) \
				-ybottom $Ymin($widget) -ytop $Ymax($widget) \
				-mbottom 20 -mleft 70 -mright 5 -mtop 5
		$can create viewport $viewa 0 $catop -height $caheight -width $cawidth -tag $AmplotTag($widget) -fill \#fef0e4 \
				-fill_frame \#f0f0ff \
				-xmode $Xlog($widget) -xleft $Fmin($widget) -xright $Fmax($widget) \
				-ymode $Ylog($widget) -ybottom $Amin($widget) -ytop $Amax($widget) \
				-mbottom 50 -mleft 70 -mright 5 -mtop 5
	
		if {$Scg($widget) != ""} {
			$can create gpl $viewt -scg $Scg($widget) -tag $TrTag($widget) -again 0.8
		}
		$can create grid $viewt -tag $TrTag($widget) -visible_y 0 -visible_ysmall 0 -visible_xsmall 0 \
						-linewidth_small 0
		$can create axes $viewt -xlabel "" -ylabel "$sta $chan" -tic_style "" -yformat none \
 						-tag $TrTag($widget) -font_numbers {Helvetica 8} -linewidth_small -1 -axis_style ""

		switch $units {
		pdbm	{
				switch $prsptype {
				A	{
						set ylabel "Acceleration PSD (pdb(m**2/s**4/hz))"
					}
				V	{
						set ylabel "Velocity PSD (pdb(m**2/s**2/hz))"
					}
				default	{
						set ylabel "Displacement PSD (pdb(m**2/hz))"
					}
				}
			}
		pdbc	{
				set ylabel "Counts PSD (pdb(cts**2/hz))"
			}
		pdbv	{
				set ylabel "Volts PSD (pdb(volt**2/hz))"
			}
		pdbuv	{
				set ylabel "Micro-volts PSD (pdb(microvolt**2/hz))"
			}
		pdbnm	{
				switch $prsptype {
				A	{
						set ylabel "Acceleration PSD (pdb(nm**2/s**4/hz))"
					}
				V	{
						set ylabel "Velocity PSD (pdb(nm**2/s**2/hz))"
					}
				default	{
						set ylabel "Displacement PSD (pdb(nm**2/hz))"
					}
				}
			}
		pm	{
				switch $prsptype {
				A	{
						set ylabel "Acceleration PSD (m**2/s**4/hz)"
					}
				V	{
						set ylabel "Velocity PSD (m**2/s**2/hz)"
					}
				default	{
						set ylabel "Displacement PSD (m**2/hz)"
					}
				}
			}
		default	{
				switch $prsptype {
				A	{
						set ylabel "Acceleration PSD (nm**2/s**4/hz)"
					}
				V	{
						set ylabel "Velocity PSD (nm**2/s**2/hz)"
					}
				default	{
						set ylabel "Displacement PSD (nm**2/hz)"
					}
				}
			}
		}
	
		$can create grid $viewa -tag $AmTag($widget)
		if {$rsptype != "C"} {
			$can create polyline $viewa -vector $p -outline red -tag $AmTag($widget) -linewidth 2
			$can create polyline $viewa -vector $p:1 -outline \#00a000 -tag $AmTag($widget) -linewidth 2
		}
		$can create polyline $viewa -vector $a:1 -outline darkblue -tag $AmTag($widget) -linewidth 0
		$can create polyline $viewa -vector $a:2 -outline darkblue -tag $AmTag($widget) -linewidth 0
		$can create polyline $viewa -vector $a -outline darkblue -tag $AmTag($widget) -linewidth 2
		$can create axes $viewa -xlabel "Frequency (Hz)" -ylabel "$ylabel" -tag $AmTag($widget) -font_numbers {Helvetica 11}
	
		$can create rectangle 0 0 [expr $cwidth-1] [expr $cheight-1]
	
	}

	proc destroy {widget} {
		variable VectorTag
		variable Scg

		::destroy $widget
		vector_free -tag $VectorTag($widget)
		if {$Scg($widget) != ""} {destroy_scg $Scg($widget)}
		destroy_dialog $widget
	}

	proc make_dialog {widget} {
		variable Instance
		variable Stachan
		variable Tmin
		variable Tmax
		variable Fmin
		variable Fmax
		variable Amin
		variable Amax
		variable Pmin
		variable Pmax
		variable Emin
		variable Emax
		variable Rsptype
		variable Xlog
		variable Ylog
		variable Canvas
		variable DialogTop

		set DialogTop($widget) [format "%s_dialog" $widget]
		toplevel $DialogTop($widget)
		wm protocol $DialogTop($widget) WM_DELETE_WINDOW "displaynoise::destroy_dialog $widget"
		wm title $DialogTop($widget) [format "displaynoise%d options %s" $Instance $Stachan($widget)]
		wm resizable $DialogTop($widget) false false
		set fr1 [format "%s.f1" $DialogTop($widget)]
		set fr2 [format "%s.f2" $DialogTop($widget)]
	
		frame $fr1
		frame $fr2
	
		button $fr1.done -text Dismiss -command "displaynoise::destroy_dialog $widget" -height 1 -pady 0
		grid $fr1.done -row 0 -column 0 -padx 0 -pady 0 -sticky w
	
		message $fr1.tmin -text tmin -width 40
		entry $fr1.tmin_entry -textvariable displaynoise::Tmin($widget) -width 15
		bind $fr1.tmin_entry <FocusOut> "+displaynoise::redraw $widget $Canvas($widget)"
		bind $fr1.tmin_entry <Return> "+displaynoise::redraw $widget $Canvas($widget)"
		message $fr1.tmax -text tmax -width 40
		entry $fr1.tmax_entry -textvariable displaynoise::Tmax($widget) -width 15
		bind $fr1.tmax_entry <FocusOut> "+displaynoise::redraw $widget $Canvas($widget)"
		bind $fr1.tmax_entry <Return> "+displaynoise::redraw $widget $Canvas($widget)"
		message $fr1.fmin -text fmin -width 40
		entry $fr1.fmin_entry -textvariable displaynoise::Fmin($widget) -width 15
		bind $fr1.fmin_entry <FocusOut> "+displaynoise::redraw $widget $Canvas($widget)"
		bind $fr1.fmin_entry <Return> "+displaynoise::redraw $widget $Canvas($widget)"
		message $fr1.fmax -text fmax -width 40
		entry $fr1.fmax_entry -textvariable displaynoise::Fmax($widget) -width 15
		bind $fr1.fmax_entry <FocusOut> "+displaynoise::redraw $widget $Canvas($widget)"
		bind $fr1.fmax_entry <Return> "+displaynoise::redraw $widget $Canvas($widget)"
		message $fr1.amin -text amin -width 40
		entry $fr1.amin_entry -textvariable displaynoise::Amin($widget) -width 15
		bind $fr1.amin_entry <FocusOut> "+displaynoise::redraw $widget $Canvas($widget)"
		bind $fr1.amin_entry <Return> "+displaynoise::redraw $widget $Canvas($widget)"
		message $fr1.amax -text amax -width 40
		entry $fr1.amax_entry -textvariable displaynoise::Amax($widget) -width 15
		bind $fr1.amax_entry <FocusOut> "+displaynoise::redraw $widget $Canvas($widget)"
		bind $fr1.amax_entry <Return> "+displaynoise::redraw $widget $Canvas($widget)"
		message $fr2.fmode -text "frequency mode" -width 120
		radiobutton $fr2.flog -text log -variable displaynoise::Xlog($widget) -value log -anchor w -command "displaynoise::redraw $widget $Canvas($widget)"
		radiobutton $fr2.flin -text linear -variable displaynoise::Xlog($widget) -value lin -anchor w -command "displaynoise::redraw $widget $Canvas($widget)"
		if {$Rsptype($widget) != "counts"} {
			message $fr2.rsptype -text "response type" -width 120
			radiobutton $fr2.displ -text Displ. -variable displaynoise::Rsptype($widget) -value displacement -anchor w -command "displaynoise::setrsptype $widget $Canvas($widget)"
			radiobutton $fr2.vel -text Vel. -variable displaynoise::Rsptype($widget) -value velocity -anchor w -command "displaynoise::setrsptype $widget $Canvas($widget)"
			radiobutton $fr2.acc -text Accel. -variable displaynoise::Rsptype($widget) -value acceleration -anchor w -command "displaynoise::setrsptype $widget $Canvas($widget)"
		}

		grid $fr1.tmin 		-row 1 -column 0 -sticky w -padx 0 -pady 0
		grid $fr1.tmin_entry 	-row 1 -column 1 -sticky w -padx 0 -pady 0
		grid $fr1.tmax 		-row 1 -column 2 -sticky w -padx 0 -pady 0
		grid $fr1.tmax_entry 	-row 1 -column 3 -sticky w -padx 0 -pady 0
		grid $fr1.fmin 		-row 2 -column 0 -sticky w -padx 0 -pady 0
		grid $fr1.fmin_entry 	-row 2 -column 1 -sticky w -padx 0 -pady 0
		grid $fr1.fmax 		-row 2 -column 2 -sticky w -padx 0 -pady 0
		grid $fr1.fmax_entry 	-row 2 -column 3 -sticky w -padx 0 -pady 0
		grid $fr1.amin 		-row 3 -column 0 -sticky w -padx 0 -pady 0
		grid $fr1.amin_entry 	-row 3 -column 1 -sticky w -padx 0 -pady 0
		grid $fr1.amax 		-row 3 -column 2 -sticky w -padx 0 -pady 0
		grid $fr1.amax_entry 	-row 3 -column 3 -sticky w -padx 0 -pady 0

		grid $fr2.fmode		-row 0 -column 0 -sticky w -padx 0 -pady 0
		grid $fr2.flog 		-row 0 -column 1 -sticky w -padx 0 -pady 0
		grid $fr2.flin 		-row 0 -column 2 -sticky w -padx 0 -pady 0
		if {$Rsptype($widget) != "counts"} {
			grid $fr2.rsptype 	-row 2 -column 0 -sticky w -padx 0 -pady 0
			grid $fr2.displ 	-row 2 -column 1 -sticky w -padx 0 -pady 0
			grid $fr2.vel 		-row 2 -column 2 -sticky w -padx 0 -pady 0
			grid $fr2.acc 		-row 2 -column 3 -sticky w -padx 0 -pady 0
		}
	
		grid $fr1 -row 0 -column 0 -sticky ew
		grid columnconfigure $fr1 0 -weight 1
		grid columnconfigure $fr1 1 -weight 1
		grid columnconfigure $fr1 2 -weight 1
		grid columnconfigure $fr1 3 -weight 1
	
		grid $fr2 -row 1 -column 0 -sticky ew
		grid columnconfigure $fr1 0 -weight 1
		grid columnconfigure $fr1 1 -weight 1
		grid columnconfigure $fr1 2 -weight 1
		grid columnconfigure $fr1 3 -weight 1
	}

	proc destroy_dialog {widget} {
		variable DialogTop

		if {[info exists DialogTop($widget)] != 0} {
			::destroy $DialogTop($widget)
			if {[info exists DialogTop($widget)] != 0} {unset DialogTop($widget)}
		}
	}

	proc line_split {width number} {
		set delta [expr double($width)/double($number)]
		set last [expr 0.5*$delta]
		set out [list [expr round($last)]]
		loop i 1 $number {
			set last [expr $last + $delta]
			lvarcat out [expr round($last)]
		}
		
		return $out
	}

	proc make_spectrum {widget rsptype rspfile prsptype units calibl} {
		variable VectorTag 

		set loff 0
		for_file line $rspfile {
			if {$line == ""} {incr loff; continue}
			if {[crange $line 0 0] == "\#"} {incr loff; continue}
			break
		}
		set ret [scan $line "%s %d %s %s" type stage auth rtype]
		if {$ret != 4} {return 0}
		set resp [read_response $rspfile]
		if {$type == "measured" && $rtype == "fap2"} {
			incr loff
			incr loff
			set a [vector_read $rspfile -tag $VectorTag($widget) -loff $loff -ypos 1 3 4]
		} else {
			set a [vector_create 3 -tag $VectorTag($widget)]
			set nx [vector_get $ain nx]
			loop i 0 $nx {
				set xy [vector_get $ain xy $i]
				set freq [lindex $xy 0]
				set rsp [eval_response $resp $freq]
				set amp [lindex $rsp 0]
				vector_append $a -1 $freq [list $amp $amp $amp]
			}
		}
		free_response $resp
		set p [vector_create 2 -tag $VectorTag($widget)]
		set nx [vector_get $a nx]
		loop i 0 $nx {
			set xy [vector_get $a xy $i]
			set freq [lindex $xy 0]
			if {[catch "peterson high $freq $prsptype" h]} continue
			if {[catch "peterson low $freq $prsptype" l]} continue
			if {$units == "pdbnm"} {
				set h [expr $h + 180.0]
				set l [expr $l + 180.0]
			}
			vector_append $p -1 $freq [list $h $l]
		}
		if {$rsptype == $prsptype} {
			switch $units {
			pdbm	{
					set nx [vector_get $a nx]
					loop i 0 $nx {
						set xy [vector_get $a xy $i]
						set ylist [list]
						set am [lindex $xy 1]
						set amh [lindex $xy 2]
						set aml [lindex $xy 3]
						set am2 [expr 10.0*[mylog10 $am]-180.0]
						set amh2 [expr 10.0*[mylog10 $amh]-180.0]
						set aml2 [expr 10.0*[mylog10 $aml]-180.0]
						lvarcat ylist $am2
						lvarcat ylist $amh2
						lvarcat ylist $aml2
						vector_puty $a $i $ylist
					}
				}
			pdbnm	{
					set nx [vector_get $a nx]
					loop i 0 $nx {
						set xy [vector_get $a xy $i]
						set ylist [list]
						set am [lindex $xy 1]
						set amh [lindex $xy 2]
						set aml [lindex $xy 3]
						set am2 [expr 10.0*[mylog10 $am]]
						set amh2 [expr 10.0*[mylog10 $amh]]
						set aml2 [expr 10.0*[mylog10 $aml]]
						lvarcat ylist $am2
						lvarcat ylist $amh2
						lvarcat ylist $aml2
						vector_puty $a $i $ylist
					}
				}
			pdbc	{
					set nx [vector_get $a nx]
					loop i 0 $nx {
						set xy [vector_get $a xy $i]
						set ylist [list]
						set am [lindex $xy 1]
						set amh [lindex $xy 2]
						set aml [lindex $xy 3]
						set am2 [expr 10.0*[mylog10 $am]]
						set amh2 [expr 10.0*[mylog10 $amh]]
						set aml2 [expr 10.0*[mylog10 $aml]]
						lvarcat ylist $am2
						lvarcat ylist $amh2
						lvarcat ylist $aml2
						vector_puty $a $i $ylist
					}
				}
			pdbv	{
					set nx [vector_get $a nx]
					loop i 0 $nx {
						set xy [vector_get $a xy $i]
						set ylist [list]
						set am [lindex $xy 1]
						set amh [lindex $xy 2]
						set aml [lindex $xy 3]
						set am2 [expr 10.0*[mylog10 [expr $am*$calibl*$calibl]]]
						set amh2 [expr 10.0*[mylog10 [expr $amh*$calibl*$calibl]]]
						set aml2 [expr 10.0*[mylog10 [expr $aml*$calibl*$calibl]]]
						lvarcat ylist $am2
						lvarcat ylist $amh2
						lvarcat ylist $aml2
						vector_puty $a $i $ylist
					}
				}
			pdbuv	{
					set nx [vector_get $a nx]
					loop i 0 $nx {
						set xy [vector_get $a xy $i]
						set ylist [list]
						set am [lindex $xy 1]
						set amh [lindex $xy 2]
						set aml [lindex $xy 3]
						set am2 [expr 10.0*[mylog10 [expr $am*$calibl*$calibl]]]
						set amh2 [expr 10.0*[mylog10 [expr $amh*$calibl*$calibl]]]
						set aml2 [expr 10.0*[mylog10 [expr $aml*$calibl*$calibl]]]
						lvarcat ylist $am2
						lvarcat ylist $amh2
						lvarcat ylist $aml2
						vector_puty $a $i $ylist
					}
				}
			pm	{
					set nx [vector_get $a nx]
					loop i 0 $nx {
						set xy [vector_get $a xy $i]
						set ylist [list]
						set am [lindex $xy 1]
						set amh [lindex $xy 2]
						set aml [lindex $xy 3]
						set am2 [expr $am*1.e-18]
						set amh2 [expr $amh*1.e-18]
						set aml2 [expr $aml*1.e-18]
						lvarcat ylist $am2
						lvarcat ylist $amh2
						lvarcat ylist $aml2
						vector_puty $a $i $ylist
					}
				}
			default	{
				}
			}
			return [list $a $p]
		}
		switch $prsptype {
		A	{
			switch $rsptype {
			A	{
				set ndiff 0.0
				}
			V	{
				set ndiff 1.0
				}
			default	{
				set ndiff 2.0
				}
			}
			}
		V	{
			switch $rsptype {
			A	{
				set ndiff -1.0
				}
			V	{
				set ndiff 0.0
				}
			default	{
				set ndiff 1.0
				}
			}
			}
		default	{
			switch $rsptype {
			A	{
				set ndiff -2.0
				}
			V	{
				set ndiff -1.0
				}
			default	{
				set ndiff 0.0
				}
			}
			}
		}
		set twopi 6.283185307
		set nx [vector_get $a nx]
		loop i 0 $nx {
			set xy [vector_get $a xy $i]
			set freq [lindex $xy 0]
			if {$ndiff != 0.0} {
				set gain [expr pow(2.0*($twopi*$freq), $ndiff)]
			} else {
				set gain 1.0
			}
			set ylist [list]
			set am [lindex $xy 1]
			set amh [lindex $xy 2]
			set aml [lindex $xy 3]
			set am2 [expr $am*$gain]
			set amh2 [expr $amh*$gain]
			set aml2 [expr $aml*$gain]
			switch $units {
			pdbm	{
					set am2 [expr 10.0*[mylog10 $am2]-180.0]
					set amh2 [expr 10.0*[mylog10 $amh2]-180.0]
					set aml2 [expr 10.0*[mylog10 $aml2]-180.0]
				}
			pdbnm	{
					set am2 [expr 10.0*[mylog10 $am2]]
					set amh2 [expr 10.0*[mylog10 $amh2]]
					set aml2 [expr 10.0*[mylog10 $aml2]]
				}
			pm	{
					set am2 [expr $am2*1.e-18]
					set amh2 [expr $amh2*1.e-18]
					set aml2 [expr $aml2*1.e-18]
				}
			default	{
				}
			}
			lvarcat ylist $am2
			lvarcat ylist $amh2
			lvarcat ylist $aml2
			vector_puty $a $i $ylist
		}
		return [list $a $p]
	}

	proc setrsptype {widget canvas} {
		variable Rsptype
		variable Prsptype
		variable Db
		variable Row
		variable Units
	
		if {$Rsptype($widget) == $Prsptype($widget)} return
		set Prsptype($widget) $Rsptype($widget)
		switch $Rsptype($widget) {
		displacement	{
				create $widget $Db($widget) $Row($widget) D $Units($widget)
				}
		velocity	{
				create $widget $Db($widget) $Row($widget) V $Units($widget)
				}
		acceleration	{
				create $widget $Db($widget) $Row($widget) A $Units($widget)
				}
		}
	}
	
	proc redraw {widget canvas} {
		variable Tmin
		variable Tmax
		variable Fmin
		variable Fmax
		variable Amin
		variable Amax
		variable Xlog
		variable Ylog
		variable VectorTag
		variable TrplotTag
		variable AmplotTag
	
		set xlog $Xlog($widget)
		set amlog $Ylog($widget)
	
		if {$xlog == "log"} {
			if {$Fmin($widget) <= 0.0} {
				set Fmin($widget) 1.e-5
			}
			if {$Fmax($widget) <= 0.0} {
				set Fmax($widget) 1.e-5
			}
		}
		if {$amlog == "log"} {
			if {$Amin($widget) <= 0.0} {
				set Amin($widget) 1.e-5
			}
			if {$Amax($widget) <= 0.0} {
				set Amax($widget) 1.e-5
			}
		}
		if {$Fmin($widget) == $Fmax($widget)} {
			if {$Fmin($widget) == 0.0} {
				set Fmin($widget) -1.0
				set Fmax($widget) 1.0
			} else {
				set Fmin($widget) [expr 0.1*$Fmin($widget)]
				set Fmax($widget) [expr 10.0*$Fmax($widget)]
			}
		}
		if {$Amin($widget) == $Amax($widget)} {
			if {$Amin($widget) == 0.0} {
				set Amin($widget) -1.0
				set Amax($widget) 1.0
			} else {
				set Amin($widget) [expr 0.1*$Amin($widget)]
				set Amax($widget) [expr 10.0*$Amax($widget)]
			}
		}
	
		$canvas itemconfigure $TrplotTag($widget) -xleft $Tmin($widget) -xright $Tmax($widget) 
		$canvas itemconfigure $AmplotTag($widget) -xleft $Fmin($widget) -xright $Fmax($widget) \
				-xmode $Xlog($widget) -ymode $Ylog($widget) -ybottom $Amin($widget) -ytop $Amax($widget)
	}
	
	proc print {widget canvas temp} {
		variable PrintInstance
		variable Psfile
	
		if {$temp == 1} {
			set fname [format "/tmp/rbc_%d_%d.ps" [pid] $PrintInstance]
		} else {
			set fname $Psfile($widget)
		}
	
		set f [open $fname w]
	
		set w [$canvas cget -width]
		set h [$canvas cget -height]
		set aspect [expr double ($w) / double ($h)]
		if {$aspect > 0.75} {
			$canvas postscript -channel $f -rotate false -pagewidth 7.5i
		} else {
			$canvas postscript -channel $f -rotate false -pageheight 10.0i
		}
		close $f
		if {$temp == 1} {
			exec lp -c -o nobanner $fname
			exec unlink $fname
		}
	
		incr PrintInstance
	}

	proc mylog10 {value} {
		if {$value <= 0.0} {
			return -1.e20
		} else {
			return [expr log10($value)]
		}
	}
}
